﻿#Requires -RunAsAdministrator
<#
* Copyright(c) 2006-2018 ThreatSTOP, Inc. All Rights Reserved
* NOTICE: All information contained herein is, and remains the property of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
* technical concepts contained herein are proprietary to ThreatSTOP, Inc. and its suppliers and certain aspects thereof are protected by United
* States Patent No. 8,533,822 and United States Patent No. 8,869,237, and are protected by trade secret or copyright law.
* US Government Entities: The ThreatSTOP service, software and documentation, as applicable, are "commercial computer software" and
* "commercial computer software documentation" developed exclusively at private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
* 12.212 or DFARS 227 7202 and their successors, as applicable, use, reproduction and disclosure of the software and documentation is
* governed by the terms of Threatstop's commercial agreement.
#>
# Version: 0.9.3 September 2016 ErrID start: 30
#
# Ok to run this multiple times. This script will install or update existing config. R
# Run on a Windows 2016 server as administrator (not ready for remote execution) uses .ini file to:
#
# 1. Check that Win DNS server is installed, add role if necessary
# 2. Prompt for TS policy/zone name, e.g. domainonly.threat73.rpz.threatstop.local.dns, save to ini file
# 3. Check if forward lookup zone is already set up in DNS, add if necessary
# 4. Make sure DNS Analytic log is enabled and set for manual truncation.
# 5. Add scheduled tasks to load policy every n minutes, export policy logs, upload policy logs, rotate policy logs

$ErrorActionPreference = "Stop"


# Due to module loading PATH issues we restart the process in 64bit if it's 32bit see: https://noise.getoto.net/tag/aws-codepipeline/

# get-module -list
if ($PSHOME -like "*SysWOW64*") {
 Write-Host -ForegroundColor Red "Win Detected 64bit pshome: $pshome"
 & (Join-Path ($PSHOME -replace "SysWOW64", "SysNative") powershell.exe) -File `
   (Join-Path $PSScriptRoot $MyInvocation.MyCommand) @args

  Exit $LastExitCode
}

# import-module servermanager # not sure we need this anymore

Write-Host -ForegroundColor Green "Win pshome: $pshome"


Write-Host "Starting ThreatSTOP DNS Policy Setup at $(Get-Date)"
#set up Windows Event Log source for ThreatSTOP
if (![System.Diagnostics.EventLog]::SourceExists("ThreatSTOP"))
{
    New-EventLog -LogName Application -Source "ThreatSTOP"
}

#Load config values into hash table:
Write-Host "Loading configuration..."
$scriptPath = split-path -parent $MyInvocation.MyCommand.Definition
Get-Content "$scriptPath\ThreatSTOP.ini" | foreach-object -begin {$h=@{}} -process { $k = [regex]::split($_,'='); if(($k[0].CompareTo("") -ne 0) -and ($k[0].StartsWith("[") -ne $True -and $k[0].StartsWith("#")  -ne $True)) { $h.Add($k[0].Trim(), $k[1].Trim()) } }
$tszonename = $h.TSZoneName

#step 2: check if DNS server is installed:
$winLabel = gwmi win32_operatingsystem | % caption
$winVersion = gwmi win32_operatingsystem version | % Version
$winReleaseID = Get-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion" -Name ReleaseId | % ReleaseId
if ((!$winLabel -Match '2016') -and ($winReleaseID -lt "1803" -and $winLabel -Match 'Windows Server'))
{
    Write-Host -ForegroundColor Red "This script must be executed on Windows Server 2016 or 2019. Exiting."
    Write-Host -ForegroundColor Red "Win Label: $winLabel"
    Write-Host -ForegroundColor Red "Win Version: $winVersion"
    Write-Host -ForegroundColor Red "Win Release ID: $winReleaseID"
    Exit 2
}

$DNSInstalled = (Get-WindowsFeature -Name "DNS").Installed
if (!$DNSInstalled)
{
    Write-Host -ForegroundColor Green "DNS Server role is not installed. Adding DNS Server role..."
    Add-WindowsFeature -Name "DNS"
}

if ($h.UseDig -eq "true")
{
	Write-Host "Using Dig. Skipping setup of secondary zone. Checking for VC runtime..."
	&"$scriptPath\scripts\VCRuntime.ps1"
}
else
{
#step 4: setup secondary zone
	#see if zone already exists:
	$zones = Get-DnsServerZone
	if (!($zones.ZoneName -Contains $tszonename))
	{
		#add zone
		Add-DnsServerSecondaryZone -Name $tszonename -ZoneFile ($tszonename + ".dns") -MasterServers $h.TSMaster
		Write-Host ($tszonename + " zone created.")
	}
	else
	{
		Write-Host ($tszonename + " zone found.")
	}
}

#step 5: enable DNS Analytic log and set retention
#need to use wevutil to enable retention, then enable log. this is a work-around for issue https://support.microsoft.com/en-us/kb/2488055
$DNSAnalyticalLog = Get-WinEvent -ListLog 'Microsoft-Windows-DNSServer/Analytical'
if ($DNSAnalyticalLog.LogMode -ne 'Retain' -or $DNSAnalyticalLog.IsEnabled -eq $false)
{
	Write-Host "Enabling DNS Analytical Log..."
    $DNSAnalyticalLog.IsEnabled = $false #must disable log before changing config.
    $DNSAnalyticalLog.SaveChanges()
    echo y | wevtutil set-log "Microsoft-Windows-DNSServer/Analytical" /retention:true /enabled:true  #this command also clears log
}

#step 6: create scheduled tasks
Write-Host "Setting up scheduled tasks..."
&"$scriptPath\scripts\createUpdateScheduledTasks.ps1"

#all done!
$msg = "ThreatSTOP DNS Policy Setup completed at $(Get-Date)"
Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Information -EventId 30 -Message $msg
Write-Host $msg
